const sqlite3 = require("sqlite3").verbose();
const DbBase = require("./db-base");
let dbs = {};

module.exports = class Sqlite3Base extends DbBase {
  constructor(dbFile, tableName, showSqlStr) {
    if (!dbFile || typeof dbFile != "string") {
      dbFile = "./sqlite.db3";
    }
    let conn = dbs[dbFile];
    if (!conn) {
      conn = new sqlite3.Database(dbFile);
      dbs[dbFile] = conn;
    }
    this.conn = conn;
    super(tableName, showSqlStr);
  }

  query(sqlStr, params) {
    if (this.showSqlStr) {
      console.log(sqlStr, params);
    }
    return new Promise((resolve, reject) => {
      if (sqlStr.indexOf("??") != -1) {
        sqlStr = sqlStr.replace("??", params[0]);
        params = params.slice(1);
      }
      if (sqlStr.indexOf("select") == 0) {
        this.conn.all(sqlStr, params, function (err, data) {
          if (err) {
            reject(err);
          } else {
            resolve(data);
          }
        });
      } else {
        this.conn.run(sqlStr, params, function (err) {
          if (err) {
            reject(err);
          } else {
            resolve(this);
          }
        });
      }
    });
  }

  insert(entity) {
    const { keys, sp, values } = formatEntity(entity);
    let sql =
      "insert into " +
      this.tableName +
      " (" +
      keys.join(",") +
      ") values (" +
      sp.join(",") +
      ")";
    return this.query(sql, values);
  }

  truncate(tableName) {
    if (!tableName) {
      tableName = this.tableName;
    }
    let sql = "delete from " + tableName;
    this.query(sql);
    let sql2 =
      "UPDATE sqlite_sequence SET seq = 0 WHERE name = '" + tableName + "'";
    return this.query(sql2);
  }

  update(entity, whereStr) {
    const { values, keystr } = formatEntity(entity);
    let sql =
      "update " + this.tableName + " set " + keystr + " where " + whereStr;
    return this.query(sql, values);
  }

  updateByParams(entity, where) {
    let { values, keystr } = formatEntity(entity);
    const t = formatEntity(where, " and ");
    values = values.concat(t.values);
    let sql =
      "update " + this.tableName + " set " + keystr + " where " + t.keystr;

    return this.query(sql, values);
  }

  updateById(entity, id) {
    const { values, keystr } = formatEntity(entity);
    values.push(id);
    let sql = "update " + this.tableName + " set " + keystr + " where id=?";
    return this.query(sql, values);
  }
  findOne(query) {
    const { values, keystr } = formatEntity(query, " and ");
    let sql =
      "select * from " +
      this.tableName +
      " where " +
      keystr +
      " order by id desc limit 1";
    return this.query(sql, values);
  }

  findWithQuery(query) {
    const { values, keystr } = formatEntity(query);
    let sql = "select * from " + this.tableName + " where " + keystr;
    return this.query(sql, values);
  }

  getTableNames() {
    let sql = "select name as table_name from sqlite_master where type='table'";
    return this.query(sql);
  }

  findColumnName(tableName) {
    let tn = tableName || this.tableName;
    let sql =
      "select * from sqlite_master where type='table' and name='" + tn + "'";
    return this.query(sql);
  }
};

function formatEntity(entity, separate) {
  let keys = [], // ["a","b","c"]
    sp = [], // ?,?
    values = [], //[1,2,3]
    keyarr = []; // a=?,b=?
  Object.keys(entity).forEach((k) => {
    keys.push(k);
    sp.push("?");
    values.push(entity[k]);
    keyarr.push(k + "=?");
  });
  let keystr = keyarr.join(separate || ",");
  return { keys, sp, values, keystr };
}
